
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Meta = imports.gi.Meta;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const MEDIA_KEYS_SCHEMA = 'org.gnome.settings-daemon.plugins.media-keys';
const CUSTOM_BINDINGS_SCHEMA = MEDIA_KEYS_SCHEMA + '.custom-keybinding';

const MY_SCHEMA = 'org.gnome.shell.extensions.command-keybindings';

function getSettings(extension) {
  const GioSSS = Gio.SettingsSchemaSource;

  // check if this extension was built with "make zip-file", and thus
  // has the schema files in a subfolder
  // otherwise assume that extension has been installed in the
  // same prefix as gnome-shell (and therefore schemas are available
  // in the standard folders)
  let schemaDir = extension.dir.get_child('schemas');
  let schemaSource;
  if (schemaDir.query_exists(null))
    schemaSource = GioSSS.new_from_directory(schemaDir.get_path(),
                                             GioSSS.get_default(),
                                             false);
  else
    schemaSource = GioSSS.get_default();

  let schemaObj = schemaSource.lookup(MY_SCHEMA, true);
  if (!schemaObj)
    throw new Error('Schema ' + MY_SCHEMA + ' could not be found for extension ' +
                    extension.metadata.uuid + '. Please check your installation.');

  return new Gio.Settings({ settings_schema: schemaObj});
}


let gsd_settings = null;
let settings = null;
let gsd_id = 0;

let paths_to_keys = {};

function init() {
    gsd_settings = new Gio.Settings({ schema: MEDIA_KEYS_SCHEMA });
    settings = getSettings(Me)
    settings.delay();
}

function bind_key(path) {
    let csettings = new Gio.Settings({schema: CUSTOM_BINDINGS_SCHEMA,
    					  path: path});
    let binding = csettings.get_string('binding');

    let key = '';

    for (let i = 0; i < 10; i++) {
	key = 'binding-' + p;
	for (p in paths_to_keys) {
	    if (paths_to_keys[p] == key) {
		key = '';
		break;
	    }
	}
	if (key) {
	    break;
	}
    }
    if (!key) {
	global.logError('Too many custom bindings');
	return;
    }

    settings.set_strv(key, new Array(binding));

    let keyb_handler = function () {
	let command = csettings.get_string('command');

	if (command) {
	    GLib.spawn_command_line_async(command);
	}
    };

    if (global.display.add_keybinding(key, settings,
    	 			      Meta.KeyBindingFlags.NONE,
    	 			      keyb_handler, csettings)) {
	paths_to_keys[path] = key;
	csettings.connect('changed::binding', function(ckey) {
	    binding = csettings.get_string('binding');
	    settings.set_strv(key, new Array(binding));
	});	
    } else {
	global.logError('Could not add binding');
	settings.set_strv(key, {});
    }
}

function unbind_key(path) {
    global.display.remove_keybinding(paths_to_keys[path]);
    settings.set_strv(paths_to_keys[path], new Array());
    delete paths_to_keys[path];
}

function enable() {
    let custom_paths = gsd_settings.get_strv('custom-keybindings')


    for (p in custom_paths) {
	bind_key(custom_paths[p]);
    }

    gsd_id = gsd_settings.connect('changed::custom-keybindings', function(key) {
	let tmp_paths = gsd_settings.get_strv('custom-keybindings');
	
	for (p in tmp_paths) {
	    if (!(tmp_paths[p] in paths_to_keys)) {
		bind_key(tmp_paths[p]);
	    }
	}

	for (path in paths_to_keys) {
	    let done = 0;

	    for (p in tmp_paths) {
		if (tmp_paths[p] == path) {
		    done = 1;
		    break;
		}
	    }
	    if (!done) {
		unbind_key(path);
	    }
	}
    });
}

function disable() {
    for (path in paths_to_keys) {
	unbind_key(path);
    }

    if (gsd_id != 0) {
	gsd_settings.disconnect(gsd_id);
	gsd_id = 0;
}
}
